export interface WebpackStatsModule {
  modules?: WebpackStatsModule[];
  identifier: string;
}

export interface WebpackStatsChunk {
  names: string[];
  modules: WebpackStatsModule[];
}

export interface WebpackStats {
  chunks: WebpackStatsChunk[];
}
